package eu.jonahbauer.json;

import eu.jonahbauer.json.token.JsonTokenKind;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.experimental.Accessors;
import eu.jonahbauer.json.token.JsonToken;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Java representation of a JSON boolean.
 */
@Getter
@Accessors(fluent = true)
@RequiredArgsConstructor
public enum JsonBoolean implements JsonValue, JsonToken, JsonTokenKind {
    TRUE(true),
    FALSE(false),
    ;

    /**
     * Creates a {@link JsonBoolean} wrapping the given {@code boolean} value.
     * @param bool a {@code boolean} value
     * @return a {@link JsonBoolean}
     */
    public static @NotNull JsonBoolean valueOf(boolean bool) {
        return bool ? TRUE : FALSE;
    }

    /**
     * Creates a {@link JsonBoolean} wrapping the given {@link Boolean} value.
     * @param bool a {@link Boolean} value
     * @return a {@link JsonBoolean}
     */
    @Contract("null -> null; !null -> !null")
    public static @Nullable JsonBoolean valueOf(@Nullable Boolean bool) {
        return bool == null ? null : valueOf((boolean) bool);
    }

    private final boolean value;

    @Override
    public @NotNull String toString() {
        return String.valueOf(value);
    }

    @Override
    public @NotNull JsonBoolean getKind() {
        return this;
    }
}
