package eu.jonahbauer.json;

import lombok.Getter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

@Getter
public class JsonProcessingException extends RuntimeException {
    private final @Nullable Position position;

    public JsonProcessingException(int fragment, int line, int column, @NotNull String message) {
        super(message);
        this.position = new Position(fragment, line, column);
    }

    public JsonProcessingException(int fragment, int line, int column, @NotNull String message, @NotNull Throwable cause) {
        super(message, cause);
        this.position = new Position(fragment, line, column);
    }

    protected JsonProcessingException(@NotNull String message) {
        super(message);
        this.position = null;
    }

    protected JsonProcessingException(@NotNull String message, @NotNull Throwable cause) {
        super(message, cause);
        this.position = null;
    }

    @Override
    public String getMessage() {
        if (position != null) {
            return super.getMessage() + " at " + position;
        } else {
            return super.getMessage();
        }
    }

    public record Position(int fragment, int line, int column) {
        public Position {
            if (fragment < -1) throw new IllegalArgumentException();
            if (line < 0) throw new IllegalArgumentException();
            if (column < 0) throw new IllegalArgumentException();
        }

        @Override
        public String toString() {
            if (fragment == -1) {
                return STR."line \{line}, column \{column}";
            } else {
                return STR."fragment \{fragment}, line \{line}, column \{column}";
            }
        }
    }
}
