package eu.jonahbauer.json.parser.tokenizer;

import eu.jonahbauer.json.token.JsonToken;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public interface JsonTokenizer extends Iterable<JsonToken> {

    @Nullable JsonToken next() throws IOException;

    int getFragment();
    int getLine();
    int getColumn();

    boolean hasTemplateSupport();


    @Override
    default @NotNull Iterator<@NotNull JsonToken> iterator() {
        class JsonTokenizerIterator implements Iterator<@NotNull JsonToken> {
            private JsonToken next;
            private boolean valid = false;


            private void ensureValid() {
                try {
                    if (!valid) {
                        next = JsonTokenizer.this.next();
                        valid = true;
                    }
                } catch (IOException ex) {
                    throw new UncheckedIOException(ex);
                }
            }

            @Override
            public boolean hasNext() {
                ensureValid();
                return next != null;
            }

            @Override
            public @NotNull JsonToken next() {
                ensureValid();
                if (next == null) {
                    throw new NoSuchElementException();
                } else {
                    valid = false;
                    return next;
                }
            }
        }

        return new JsonTokenizerIterator();
    }

    default @NotNull Stream<@NotNull JsonToken> stream() {
        return StreamSupport.stream(this.spliterator(), false);
    }
}
