package eu.jonahbauer.json.parser.tokenizer;

import org.junit.jupiter.api.Test;

import java.io.IOException;

import static java.lang.StringTemplate.RAW;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class JsonTokenizerImpl$TemplateReaderTest {

    @Test
    void readString() throws IOException {
        var tokenizer = new TemplateReader("Hello World!");
        assertEquals('H', tokenizer.read());
        assertEquals('e', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals(' ', tokenizer.read());
        assertEquals('W', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals('r', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('d', tokenizer.read());
        assertEquals('!', tokenizer.read());
        assertEquals(-1, tokenizer.read());
        assertEquals(-1, tokenizer.read());
    }

    @Test
    void readStringTemplate() throws IOException {
        String name = "World";
        var tokenizer = new TemplateReader(RAW."Hello \{name}!");
        assertEquals('H', tokenizer.read());
        assertEquals('e', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals(' ', tokenizer.read());
        assertEquals(-2, tokenizer.read());
        assertEquals(name, tokenizer.getObject());
        assertEquals('!', tokenizer.read());
        assertThrows(IllegalStateException.class, tokenizer::getObject);
        assertEquals(-1, tokenizer.read());
        assertEquals(-1, tokenizer.read());
    }

    @Test
    void pushbackString() throws IOException {
        var tokenizer = new TemplateReader("Hello World!");
        assertEquals('H', tokenizer.read());
        assertEquals('e', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals(' ', tokenizer.read());
        tokenizer.pushback();
        assertEquals(' ', tokenizer.read());
        assertEquals('W', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals('r', tokenizer.read());
        assertEquals('l', tokenizer.read());
        tokenizer.pushback();
        assertEquals('l', tokenizer.read());
        assertEquals('d', tokenizer.read());
        assertEquals('!', tokenizer.read());
        assertEquals(-1, tokenizer.read());
        assertEquals(-1, tokenizer.read());
        tokenizer.pushback();
        assertEquals(-1, tokenizer.read());
    }

    @Test
    void pushbackPlaceholder() throws IOException {
        String name = "World";
        var tokenizer = new TemplateReader(RAW."Hello \{name}!");
        assertEquals('H', tokenizer.read());
        assertEquals('e', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('l', tokenizer.read());
        assertEquals('o', tokenizer.read());
        assertEquals(' ', tokenizer.read());
        assertEquals(-2, tokenizer.read());
        assertEquals(name, tokenizer.getObject());
        tokenizer.pushback();
        assertEquals(-2, tokenizer.read());
        assertEquals(name, tokenizer.getObject());
        assertEquals('!', tokenizer.read());
        assertThrows(IllegalStateException.class, tokenizer::getObject);
        assertEquals(-1, tokenizer.read());
        assertEquals(-1, tokenizer.read());
    }
}
